import koji

import fedmsg
import fedmsg.consumers

import logging
log = logging.getLogger('tag2distrepo')


class Tag2DistRepoConsumer(fedmsg.consumers.FedmsgConsumer):
    config_key = 'tag2distrepo.enabled'

    def __init__(self, hub):
        super(Tag2DistRepoConsumer, self).__init__(hub)

        config = hub.config
        prefix = config['topic_prefix']
        env = config.get('environment')
        self.topic = [
            '%s.%s.buildsys.tag' % (prefix, env)
        ]

        log.info('Tag2DistRepo consumer ready for service')

    def consume(self, msg):
        topic = msg['topic']
        if topic not in self.topic:
            return

        msg = msg['body']['msg']

        #  {u'build_id': 799208,
        #   u'name': u'python-fmn-rules',
        #   u'tag_id': 374,
        #   u'instance': u'primary',
        #   u'tag': u'epel7-infra',
        #   u'user': u'puiterwijk',
        #   u'version': u'0.9.1',
        #   u'owner': u'sayanchowdhury',
        #   u'release': u'1.el7'}}

        build_nvr = '%(name)s-%(version)s-%(release)s' % msg
        build_id = msg['build_id']
        tag = msg['tag']
        koji_instance = msg['instance']

        log.info('Build %s (%s) tagged into %s on %s',
                 build_nvr, build_id, tag, koji_instance)

        config = fedmsg.config.load_config()

        if koji_instance not in config['tag2distrepo.tags']:
            log.info('Koji instance %s not enabled', koji_instance)
            return
        kconfig = config['tag2distrepo.tags'][koji_instance]

        if tag not in kconfig['tags']:
            log.info('Tag %s not enabled for koji instance %s',
                     tag, koji_instance)
            return

        keys = kconfig['tags'][tag]

        log.info('Creating koji client for %s', koji_instance)
        client = koji.ClientSession(kconfig['url'],
                                    kconfig['options'])

        if kconfig['authmethod'] == 'ssl':
            client.ssl_login(kconfig['cert'], None, kconfig['serverca'])
        elif kconfig['authmethod'] == 'kerberos':
            kwargs = {}
            for opt in ('principal', 'keytab', 'ccache'):
                if opt in kconfig:
                    kwargs[opt] = kconfig[opt]
            client.krb_login(**kwargs)
        else:
            raise Exception('Only SSL and kerberos authmetods supported')

        log.info('Creating distrepo task on koji %s, tag %s, keys %s',
                 koji_instance, tag, keys)

        taginfo = client.getTag(tag)

        opts = {
            'arch': (taginfo['arches'] or '').split(),
            'comp': None,
            'delta': [],
            'event': None,
            'inherit': False,
            'latest': True,
            'multilib': False,
            'skip_missing_signatures': False,
            'allow_missing_signatures': False
        }

        task_id = client.distRepo(tag, keys, **opts)

        log.info('Task ID: %s', task_id)

        client.logout()
